@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Ricardo Fernndez Pascual
+ *  Copyright (C) 2000, 2001, 2002 Ricardo Fernández Pascual
+ *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,11 +15,14 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-embed-popup-control.c,v 1.13 2003/11/22 11:32:31 marco Exp $
  */
 
 #include "ephy-embed-popup-control.h"
+#include "ephy-embed-factory.h"
+#include "ephy-embed-persist.h"
 #include "ephy-bonobo-extensions.h"
-#include "ephy-embed-utils.h"
 #include "ephy-prefs.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
@@ -26,8 +30,8 @@
 #include <gtk/gtkmain.h>
 #include <string.h>
 #include <bonobo/bonobo-ui-component.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkclipboard.h>
-#include <libgnome/gnome-exec.h>
 
 typedef enum
 {
@@ -36,6 +40,8 @@ typedef enum
 	EMBED_POPUP_ELEMENT
 } EmbedPopupType;
 
+#define EPHY_EMBED_POPUP_CONTROL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_POPUP_CONTROL, EphyEmbedPopupControlPrivate))
+
 struct EphyEmbedPopupControlPrivate
 {
 	EphyEmbedEvent *event;
@@ -242,12 +248,15 @@ ephy_embed_popup_control_class_init (EphyEmbedPopupControlClass *klass)
                                                               "Bonobo control",
                                                               BONOBO_TYPE_CONTROL,
                                                               G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof(EphyEmbedPopupControlPrivate));
 }
 
 static void
 ephy_embed_popup_control_init (EphyEmbedPopupControl *gep)
 {
-        gep->priv = g_new0 (EphyEmbedPopupControlPrivate, 1);
+        gep->priv = EPHY_EMBED_POPUP_CONTROL_GET_PRIVATE (gep);
+
 	gep->priv->control = NULL;
 	gep->priv->embed = NULL;
 	gep->priv->event = NULL;
@@ -257,14 +266,7 @@ ephy_embed_popup_control_init (EphyEmbedPopupControl *gep)
 static void
 ephy_embed_popup_control_finalize (GObject *object)
 {
-	EphyEmbedPopupControl *gep;
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_EPHY_EMBED_POPUP_CONTROL (object));
-
-        gep = EPHY_EMBED_POPUP_CONTROL (object);
-
-        g_return_if_fail (gep->priv != NULL);
+	EphyEmbedPopupControl *gep = EPHY_EMBED_POPUP_CONTROL (object);
 
 	if (gep->priv->event)
 	{
@@ -273,8 +275,6 @@ ephy_embed_popup_control_finalize (GObject *object)
 
 	g_free (gep->priv->selection);
 
-        g_free (gep->priv);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -290,11 +290,9 @@ ephy_embed_popup_control_new (BonoboControl *control)
 {
 	EphyEmbedPopupControl *p;
 
-        p = EPHY_EMBED_POPUP_CONTROL (g_object_new (EPHY_EMBED_POPUP_CONTROL_TYPE,
-						      "BonoboControl", control,
-						      NULL));
-
-        g_return_val_if_fail (p->priv != NULL, NULL);
+	p = EPHY_EMBED_POPUP_CONTROL (g_object_new (EPHY_TYPE_EMBED_POPUP_CONTROL,
+						    "BonoboControl", control,
+						    NULL));
 
         return p;
 }
@@ -393,7 +391,7 @@ ephy_embed_popup_control_set_event (EphyEmbedPopupControl *p,
 		g_object_unref (G_OBJECT (p->priv->event));
 	}
 
-	ephy_embed_event_get_context (event, &context);
+	context = ephy_embed_event_get_context (event);
 
 	p->priv->context = context;
 
@@ -433,7 +431,7 @@ ephy_embed_popup_control_connect_verbs (EphyEmbedPopupControl *p,
 EphyEmbedEvent *
 ephy_embed_popup_control_get_event (EphyEmbedPopupControl *p)
 {
-	g_return_val_if_fail (IS_EPHY_EMBED_POPUP_CONTROL (p), NULL);
+	g_return_val_if_fail (EPHY_IS_EMBED_POPUP_CONTROL (p), NULL);
 
 	return p->priv->event;
 }
@@ -444,7 +442,7 @@ embed_popup_copy_location_cmd (BonoboUIComponent *uic,
                                const char* verbname)
 {
 	char *location;
-	ephy_embed_get_location (popup->priv->embed, FALSE, &location);
+	location = ephy_embed_get_location (popup->priv->embed, FALSE);
 	embed_popup_copy_to_clipboard (popup, location);
 	g_free (location);
 }
@@ -479,41 +477,47 @@ embed_popup_copy_link_location_cmd (BonoboUIComponent *uic,
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
+static void
+save_url (EphyEmbedPopupControl *popup,
+	  const char *title,
+	  gboolean ask_dest,
+	  const char *location)
+{
+	GtkWidget *widget, *window;
+	EphyEmbedPersist *persist;
+
+	widget = GTK_WIDGET (popup->priv->embed);
+	window = gtk_widget_get_toplevel (widget);
+
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+
+	ephy_embed_persist_set_fc_title (persist, title);
+	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	ephy_embed_persist_set_flags
+		(persist, ask_dest ? EMBED_PERSIST_ASK_DESTINATION : 0);
+	ephy_embed_persist_set_persist_key
+		(persist, CONF_STATE_DOWNLOAD_DIR);
+	ephy_embed_persist_set_source (persist, location);
+
+	ephy_embed_persist_save (persist);
+}
+
 static void
 save_property_url (EphyEmbedPopupControl *popup,
+		   const char *title,
 		   gboolean ask_dest,
-		   gboolean show_progress,
 		   const char *property)
 {
 	EphyEmbedEvent *info;
 	const char *location;
 	const GValue *value;
-	GtkWidget *widget;
-	GtkWidget *window;
-	EphyEmbedPersist *persist;
 
 	info = ephy_embed_popup_control_get_event (popup);
 	ephy_embed_event_get_property (info, property, &value);
 	location = g_value_get_string (value);
 
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-
-	persist = ephy_embed_persist_new (popup->priv->embed);
-
-	ephy_embed_persist_set_source (persist, location);
-
-	if (show_progress)
-	{
-		ephy_embed_persist_set_flags (persist,
-					      EMBED_PERSIST_SHOW_PROGRESS);
-	}
-
-	ephy_embed_utils_save (window,
-			       CONF_STATE_DOWNLOADING_DIR,
-			       ask_dest,
-                               FALSE,
-                               persist);
+	save_url (popup, title, ask_dest, location);
 }
 
 /* commands */
@@ -540,9 +544,9 @@ embed_popup_download_link_cmd (BonoboUIComponent *uic,
                                const char* verbname)
 {
 	save_property_url (popup,
-		           eel_gconf_get_boolean
-		           (CONF_ASK_DOWNLOAD_DEST),
-		           TRUE, "link");
+			   _("Download Link"),
+			   FALSE,
+		           "link");
 }
 
 static void
@@ -550,7 +554,7 @@ embed_popup_save_image_as_cmd (BonoboUIComponent *uic,
 			       EphyEmbedPopupControl *popup,
                                const char* verbname)
 {
-	save_property_url (popup, TRUE, FALSE, "image");
+	save_property_url (popup, _("Save Image As"), TRUE, "image");
 }
 
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
@@ -563,16 +567,14 @@ background_download_completed (EphyEmbedPersist *persist,
 	const char *bg;
 	char *type;
 
-	ephy_embed_persist_get_dest (persist, &bg);
+	bg = ephy_embed_persist_get_dest (persist);
 	eel_gconf_set_string (CONF_DESKTOP_BG_PICTURE, bg);
 
 	type = eel_gconf_get_string (CONF_DESKTOP_BG_TYPE);
-	if (type || strcmp (type, "none") == 0)
+	if (type == NULL || strcmp (type, "none") == 0)
 	{
-		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE,
-				      "wallpaper");
+		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE, "wallpaper");
 	}
-
 	g_free (type);
 
 	g_object_unref (persist);
@@ -593,14 +595,15 @@ embed_popup_set_image_as_background_cmd (BonoboUIComponent *uic,
 	ephy_embed_event_get_property (info, "image", &value);
 	location = g_value_get_string (value);
 
-	persist = ephy_embed_persist_new (popup->priv->embed);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
 	base = g_path_get_basename (location);
-	dest = g_build_filename (ephy_dot_dir (),
-				 base, NULL);
+	dest = g_build_filename (ephy_dot_dir (), base, NULL);
 
-	ephy_embed_persist_set_source (persist, location);
 	ephy_embed_persist_set_dest (persist, dest);
+	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_source (persist, location);
 
 	ephy_embed_persist_save (persist);
 
@@ -627,35 +630,6 @@ embed_popup_copy_image_location_cmd (BonoboUIComponent *uic,
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
-static void
-save_url (EphyEmbedPopupControl *popup,
-	  gboolean ask_dest,
-	  gboolean show_progress,
-	  const char *url)
-{
-	GtkWidget *widget;
-	GtkWidget *window;
-	EphyEmbedPersist *persist;
-
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-
-	persist = ephy_embed_persist_new (popup->priv->embed);
-	ephy_embed_persist_set_source (persist, url);
-
-	if (show_progress)
-	{
-		ephy_embed_persist_set_flags (persist,
-						EMBED_PERSIST_SHOW_PROGRESS);
-	}
-
-	ephy_embed_utils_save (window,
-			       CONF_STATE_DOWNLOADING_DIR,
-			       ask_dest,
-                               FALSE,
-                               persist);
-}
-
 static void
 embed_popup_save_page_as_cmd (BonoboUIComponent *uic,
 			      EphyEmbedPopupControl *popup,
@@ -663,9 +637,8 @@ embed_popup_save_page_as_cmd (BonoboUIComponent *uic,
 {
 	char *location;
 
-	ephy_embed_get_location (popup->priv->embed,
-				   FALSE, &location);
-	save_url (popup, TRUE, FALSE, location);
+	location = ephy_embed_get_location (popup->priv->embed, FALSE);
+	save_url (popup, _("Save Page As"), TRUE, location);
 	g_free (location);
 }
 
@@ -674,7 +647,8 @@ embed_popup_save_background_as_cmd (BonoboUIComponent *uic,
 				    EphyEmbedPopupControl *popup,
                                     const char* verbname)
 {
-	save_property_url (popup, TRUE, FALSE, "background_image");
+	save_property_url (popup, _("Save Background As"),
+			   TRUE, "background_image");
 }
 
 static void
@@ -684,8 +658,7 @@ embed_popup_open_frame_cmd (BonoboUIComponent *uic,
 {
 	char *location;
 
-	ephy_embed_get_location (popup->priv->embed,
-				   FALSE, &location);
+	location = ephy_embed_get_location (popup->priv->embed, FALSE);
 
 	ephy_embed_load_url (popup->priv->embed, location);
 
