@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encodings.c,v 1.4 2003/11/30 14:10:15 chpe Exp $
+ *  $Id: ephy-encodings.c,v 1.7 2003/12/22 14:05:17 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -259,7 +259,6 @@ ephy_encodings_get_encodings (EphyEncodings *encodings,
 			list = g_list_prepend (list, kid);
 		}
 	}
-	ephy_node_thaw (encodings->priv->encodings);
 
 	return list;
 }
@@ -295,6 +294,7 @@ ephy_encodings_add_recent (EphyEncodings *encodings,
 				       (GCompareFunc) strcmp);
 	if (element != NULL)
 	{
+		g_free (element->data);
 		encodings->priv->recent =
 			g_slist_remove_link (encodings->priv->recent, element);
 	}
@@ -458,6 +458,15 @@ ephy_encoding_info_free (EphyEncodingInfo *info)
 	}
 }
 
+gboolean
+ephy_encoding_info_is_automatic (EphyEncodingInfo *info)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+
+	return (info->encoding_source < EMBED_ENCODING_PARENT_FORCED)
+		&& (info->forced_encoding == NULL || info->forced_encoding[0] == '\0');
+}
+
 EphyEncodings *
 ephy_encodings_new (void)
 {
