@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.17 2003/10/29 00:08:01 marco Exp $
+ *  $Id: EventContext.cpp,v 1.18 2003/12/25 14:48:12 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -698,7 +698,7 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, EphyEmb
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aMouseEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aMouseEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -758,7 +758,7 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, EphyEmbedEven
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aKeyEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aKeyEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
