@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,28 +16,32 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.45 2003/10/13 17:38:14 chpe Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.64 2003/12/12 21:44:35 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
+#include "mozilla-embed-single.h"
+
+#include "ephy-cookie-manager.h"
+#include "ephy-password-manager.h"
+#include "ephy-permission-manager.h"
+
 #include "glib.h"
 #include "ephy-debug.h"
 #include "gtkmozembed.h"
-#include "mozilla-embed-single.h"
-#include "ephy-prefs.h"
+#include "mozilla-embed.h"
 #include "ephy-file-helpers.h"
 #include "mozilla-notifiers.h"
 #include "ephy-langs.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-embed-prefs.h"
 #include "MozRegisterComponents.h"
-#include "FilePicker.h"
+#include "EphySingle.h"
 
-#include <time.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
@@ -50,11 +55,26 @@
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
 #include <nsICookieManager.h>
+#include <nsIPassword.h>
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
+#if MOZILLA_SNAPSHOT > 9
+#include <nsICookie2.h>
+#else
 #include <nsICookie.h>
+#endif
+#if MOZILLA_SNAPSHOT > 12
+#include <nsICookieManager.h>
+#else
 #include <nsCCookieManager.h>
+#endif
 #include <nsCPasswordManager.h>
+#include <nsIPermission.h>
+#include <nsIPermissionManager.h>
+#include <nsString.h>
+#include <nsILocalFile.h>
+#include <nsIURI.h>
+#include <nsNetUtil.h>
 
 // FIXME: For setting the locale. hopefully gtkmozembed will do itself soon
 #include <nsIChromeRegistry.h>
@@ -65,55 +85,6 @@
 #define MOZILLA_PROFILE_FILE "prefs.js"
 #define DEFAULT_PROFILE_FILE SHARE_DIR"/default-prefs.js"
 
-static void
-mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass);
-static void
-mozilla_embed_single_init (MozillaEmbedSingle *ges);
-static void
-mozilla_embed_single_finalize (GObject *object);
-
-static gresult      
-impl_clear_cache (EphyEmbedSingle *shell);
-static gresult          
-impl_set_offline_mode (EphyEmbedSingle *shell,
-		       gboolean offline);
-static gresult           
-impl_load_proxy_autoconf (EphyEmbedSingle *shell,
-			  const char* url);
-static gresult
-impl_get_font_list (EphyEmbedSingle *shell,
-		    const char *langGroup,
-		    GList **fontList);
-static gresult           
-impl_list_cookies (EphyEmbedSingle *shell,
-		   GList **cookies);
-static gresult           
-impl_remove_cookies (EphyEmbedSingle *shell,
-		     GList *cookies);
-static gresult           
-impl_list_passwords (EphyEmbedSingle *shell,
-		     PasswordType type, 
-		     GList **passwords);
-static gresult           
-impl_remove_passwords (EphyEmbedSingle *shell,
-		       GList *passwords,
-		       PasswordType type);
-static gresult 
-impl_show_file_picker (EphyEmbedSingle *shell,
-		       GtkWidget *parentWidget, 
-		       const char *title,
-		       const char *directory,
-		       const char *file, 
-		       FilePickerMode mode,
-                       char **ret_fullpath, 
-                       FileFormat *file_formats, 
-		       int *ret_file_format);
-
-static void mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
-            	           		              GtkMozEmbed **retval, 
-					              guint chrome_mask,
-                           		              EphyEmbedSingle *shell);
-
 #define MOZILLA_EMBED_SINGLE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_SINGLE, MozillaEmbedSinglePrivate))
 
 struct MozillaEmbedSinglePrivate
@@ -122,16 +93,25 @@ struct MozillaEmbedSinglePrivate
 	
 	/* monitor this widget for theme changes*/
 	GtkWidget *theme_window;
+
+	EphySingle *mSingleObserver;
 };
 
+static void mozilla_embed_single_class_init	(MozillaEmbedSingleClass *klass);
+static void ephy_embed_single_iface_init	(EphyEmbedSingleClass *iface);
+static void ephy_cookie_manager_iface_init	(EphyCookieManagerIFace *iface);
+static void ephy_password_manager_iface_init	(EphyPasswordManagerIFace *iface);
+static void ephy_permission_manager_iface_init	(EphyPermissionManagerIFace *iface);
+static void mozilla_embed_single_init		(MozillaEmbedSingle *ges);
+
 static GObjectClass *parent_class = NULL;
 
 GType
 mozilla_embed_single_get_type (void)
 {
-       static GType mozilla_embed_single_type = 0;
+       static GType type = 0;
 
-        if (mozilla_embed_single_type == 0)
+        if (type == 0)
         {
                 static const GTypeInfo our_info =
                 {
@@ -146,41 +126,57 @@ mozilla_embed_single_get_type (void)
                         (GInstanceInitFunc) mozilla_embed_single_init
                 };
 
-                mozilla_embed_single_type = g_type_register_static (EPHY_TYPE_EMBED_SINGLE,
-								   "MozillaEmbedSingle",
-								   &our_info, (GTypeFlags)0);
-        }
+		static const GInterfaceInfo embed_single_info =
+		{
+			(GInterfaceInitFunc) ephy_embed_single_iface_init,
+			NULL,
+			NULL
+		};
 
-        return mozilla_embed_single_type;
-}
+		static const GInterfaceInfo cookie_manager_info =
+		{
+			(GInterfaceInitFunc) ephy_cookie_manager_iface_init,
+			NULL,
+			NULL
+		};
 
-static void
-mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	EphyEmbedSingleClass *shell_class = EPHY_EMBED_SINGLE_CLASS (klass);
-	
-	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
-	
-        object_class->finalize = mozilla_embed_single_finalize;
+		static const GInterfaceInfo password_manager_info =
+		{
+			(GInterfaceInitFunc) ephy_password_manager_iface_init,
+			NULL,
+			NULL
+		};
 
-	shell_class->clear_cache = impl_clear_cache;
-	shell_class->set_offline_mode = impl_set_offline_mode;
-	shell_class->load_proxy_autoconf = impl_load_proxy_autoconf;
-	shell_class->get_font_list = impl_get_font_list;
-	shell_class->list_cookies = impl_list_cookies;
-	shell_class->remove_cookies = impl_remove_cookies;
-	shell_class->list_passwords = impl_list_passwords;
-	shell_class->remove_passwords = impl_remove_passwords;
-	shell_class->show_file_picker = impl_show_file_picker;
+		static const GInterfaceInfo permission_manager_info =
+		{
+			(GInterfaceInitFunc) ephy_permission_manager_iface_init,
+			NULL,
+			NULL
+		};
 
-	g_type_class_add_private (object_class, sizeof(MozillaEmbedSinglePrivate));
-}
+		type = g_type_register_static (G_TYPE_OBJECT,
+					       "MozillaEmbedSingle",
+					       &our_info,
+					       (GTypeFlags)0);
 
-EphyEmbedSingle *
-mozilla_embed_single_new (void)
-{
-	return EPHY_EMBED_SINGLE (g_object_new (MOZILLA_TYPE_EMBED_SINGLE, NULL));
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_EMBED_SINGLE,
+					     &embed_single_info);
+
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_COOKIE_MANAGER,
+					     &cookie_manager_info);
+
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_PASSWORD_MANAGER,
+					     &password_manager_info);
+
+		g_type_add_interface_static (type,
+					     EPHY_TYPE_PERMISSION_MANAGER,
+					     &permission_manager_info);
+	}
+
+        return type;
 }
 
 static gboolean
@@ -290,6 +286,17 @@ mozilla_setup_colors (MozillaEmbedSingle *mes)
 	mes->priv->theme_window = window;
 }			
 
+static void 
+mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
+                      		          GtkMozEmbed **retval, 
+					  guint chrome_mask,
+                           		  EphyEmbedSingle *shell)
+{
+ 	g_assert (chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME);
+
+	*retval = _mozilla_embed_new_xul_dialog ();
+}
+
 static void
 mozilla_init_single (MozillaEmbedSingle *mes)
 {	
@@ -304,7 +311,7 @@ mozilla_init_single (MozillaEmbedSingle *mes)
 
         /* allow creation of orphan windows */
         g_signal_connect (G_OBJECT (single), "new_window_orphan",
-                          GTK_SIGNAL_FUNC (mozilla_embed_single_new_window_orphan_cb),
+                          G_CALLBACK (mozilla_embed_single_new_window_orphan_cb),
 			  mes);
 }
 
@@ -364,22 +371,6 @@ mozilla_register_external_protocols (void)
 	{
 		mozilla_unregister_FtpProtocolHandler ();
 	}
-
-	mozilla_register_MailtoProtocolHandler ();
-}
-
-static void
-mozilla_embed_single_init (MozillaEmbedSingle *mes)
-{
- 	mes->priv = MOZILLA_EMBED_SINGLE_GET_PRIVATE (mes);
-
-	mes->priv->theme_window = NULL;
-	mes->priv->user_prefs =
-		g_build_filename (ephy_dot_dir (), 
-				  MOZILLA_PROFILE_DIR,
-				  MOZILLA_PROFILE_NAME,
-				  MOZILLA_PROFILE_FILE,
-				  NULL);
 }
 
 static nsresult
@@ -394,9 +385,14 @@ getUILang (nsAString& aUILang)
 		return NS_ERROR_FAILURE;
 	}
 
+#if MOZILLA_SNAPSHOT >= 12
+	result = localeService->GetLocaleComponentForUserAgent (aUILang);
+#else
 	nsXPIDLString uiLang;
 	result = localeService->GetLocaleComponentForUserAgent (getter_Copies(uiLang));
-	aUILang = uiLang;
+	aUILang.Assign (uiLang);
+#endif
+
 	if (NS_FAILED (result))
 	{
 		g_warning ("Could not determine locale!\n");
@@ -435,8 +431,19 @@ mozilla_init_chrome (void)
 	return NS_OK;
 }
 
-gboolean
-mozilla_embed_single_init_services (MozillaEmbedSingle *single)
+static void
+mozilla_init_observer (MozillaEmbedSingle *single)
+{
+	single->priv->mSingleObserver = new EphySingle ();
+
+	if (single->priv->mSingleObserver)
+	{
+		single->priv->mSingleObserver->Init (EPHY_EMBED_SINGLE (single));
+	}
+}
+
+static gboolean
+init_services (MozillaEmbedSingle *single)
 {
 	/* Pre initialization */
 	mozilla_init_home ();
@@ -468,52 +475,42 @@ mozilla_embed_single_init_services (MozillaEmbedSingle *single)
 
 	mozilla_register_external_protocols ();
 
+	mozilla_init_observer (single);
+
 	return TRUE;
 }
 
-static void 
-mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
-                      		          GtkMozEmbed **retval, 
-					  guint chrome_mask,
-                           		  EphyEmbedSingle *shell)
+static void
+mozilla_embed_single_init (MozillaEmbedSingle *mes)
 {
-	/* FIXME conversion duped in mozilla_embed */
-	EphyEmbed *new_embed;
-	int i;
-        EmbedChromeMask mask = EMBED_CHROME_OPENASPOPUP;
-        
-        struct
-        {
-                guint chromemask;
-                EmbedChromeMask embed_mask;
-        }
-        conversion_map [] =
-        {
-                { GTK_MOZ_EMBED_FLAG_DEFAULTCHROME, EMBED_CHROME_DEFAULT },
-                { GTK_MOZ_EMBED_FLAG_MENUBARON, EMBED_CHROME_MENUBARON },
-                { GTK_MOZ_EMBED_FLAG_TOOLBARON, EMBED_CHROME_TOOLBARON },
-                { GTK_MOZ_EMBED_FLAG_STATUSBARON, EMBED_CHROME_STATUSBARON },
-                { GTK_MOZ_EMBED_FLAG_WINDOWRAISED, EMBED_CHROME_WINDOWRAISED },
-                { GTK_MOZ_EMBED_FLAG_WINDOWLOWERED, EMBED_CHROME_WINDOWLOWERED },
-                { GTK_MOZ_EMBED_FLAG_CENTERSCREEN, EMBED_CHROME_CENTERSCREEN },
-                { GTK_MOZ_EMBED_FLAG_OPENASDIALOG, EMBED_CHROME_OPENASDIALOG },
-                { GTK_MOZ_EMBED_FLAG_OPENASCHROME, EMBED_CHROME_OPENASCHROME },
-                { 0, EMBED_CHROME_NONE }
-        };
+ 	mes->priv = MOZILLA_EMBED_SINGLE_GET_PRIVATE (mes);
 
-        for (i = 0; conversion_map[i].chromemask != 0; i++)
-        {
-                if (chrome_mask & conversion_map[i].chromemask)
-                {
-                        mask = (EmbedChromeMask) (mask | conversion_map[i].embed_mask); 
-                }
-        }
-       
-	g_signal_emit_by_name (shell, "new_window_orphan", &new_embed, mask);
+	mes->priv->theme_window = NULL;
+	mes->priv->user_prefs =
+		g_build_filename (ephy_dot_dir (), 
+				  MOZILLA_PROFILE_DIR,
+				  MOZILLA_PROFILE_NAME,
+				  MOZILLA_PROFILE_FILE,
+				  NULL);
 
-	g_assert (new_embed != NULL);
-	
-	*retval = GTK_MOZ_EMBED(EPHY_EMBED(new_embed));
+	mes->priv->mSingleObserver = nsnull;
+
+	if (!init_services (mes))
+	{
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new
+			(NULL,
+                         GTK_DIALOG_MODAL,
+                         GTK_MESSAGE_ERROR,
+                         GTK_BUTTONS_CLOSE,
+                         _("Epiphany can't be used now. "
+                         "Mozilla initialization failed. Check your "
+                         "MOZILLA_FIVE_HOME environmental variable."));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+
+		exit (0);
+	}
 }
 
 static void
@@ -537,57 +534,51 @@ mozilla_embed_single_finalize (GObject *object)
 	}
 }
 
-static gresult      
+static void
 impl_clear_cache (EphyEmbedSingle *shell)
 {
 	nsresult rv;
 	
 	nsCOMPtr<nsICacheService> CacheService =
                         do_GetService (NS_CACHESERVICE_CONTRACTID, &rv);
-	if (NS_FAILED(rv)) return G_FAILED;
-
-	CacheService->EvictEntries (nsICache::STORE_ANYWHERE);
-
-	return G_OK;
+	if (NS_SUCCEEDED (rv))
+	{
+		CacheService->EvictEntries (nsICache::STORE_ANYWHERE);
+	}
 }
 
-static gresult          
+static void
 impl_set_offline_mode (EphyEmbedSingle *shell,
 		       gboolean offline)
 {
 	nsresult rv;
 
 	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
-        if (NS_FAILED(rv))
-                return G_FAILED;
-
-        rv = io->SetOffline(offline);
-	if (NS_SUCCEEDED(rv)) return G_FAILED;
-	
-	return G_OK;
+	if (NS_SUCCEEDED (rv))
+	{
+		io->SetOffline(offline);
+	}
 }
 
-static gresult           
+static void
 impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 			  const char* url)
 {
-	nsresult rv;
+	g_assert (url != NULL);
 
+	nsresult rv;
         nsCOMPtr<nsIProtocolProxyService> pps =
                 do_GetService ("@mozilla.org/network/protocol-proxy-service;1",
                                &rv);
-        if (NS_FAILED(rv) || !pps) return G_FAILED;
-
-        rv = pps->ConfigureFromPAC (url);
-	if (NS_FAILED(rv)) return G_FAILED;
-	
-	return G_OK;
+	if (NS_SUCCEEDED (rv))
+	{
+		pps->ConfigureFromPAC (url);
+	}
 }
 
-static gresult
+static GList *
 impl_get_font_list (EphyEmbedSingle *shell,
-		    const char *langGroup,
-		    GList **fontList)
+		    const char *langGroup)
 {
 	nsresult rv;
 	PRUint32 fontCount;
@@ -596,11 +587,11 @@ impl_get_font_list (EphyEmbedSingle *shell,
 
 	nsCOMPtr<nsIFontEnumerator> mozFontEnumerator;
 	mozFontEnumerator = do_CreateInstance("@mozilla.org/gfx/fontenumerator;1", &rv);
-	if(NS_FAILED(rv)) return G_FAILED;
+	if(NS_FAILED(rv)) return NULL;
 
 	rv = mozFontEnumerator->EnumerateFonts (nsnull, nsnull,
 					        &fontCount, &fontArray);
-	if (NS_FAILED (rv)) return G_FAILED;
+	if (NS_FAILED (rv)) return NULL;
 
 	for (PRUint32 i = 0; i < fontCount; i++)
 	{
@@ -613,227 +604,321 @@ impl_get_font_list (EphyEmbedSingle *shell,
 
 	nsMemory::Free (fontArray);
 
-	*fontList = g_list_reverse (l);
-
-	return G_OK;
+	return g_list_reverse (l);
 }
 
-static gresult           
-impl_list_cookies (EphyEmbedSingle *shell,
-		   GList **cookies)
-{
-        nsresult result;
-
-        nsCOMPtr<nsICookieManager> cookieManager = 
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
-        nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
-        result = 
-            cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
-        if (NS_FAILED(result)) return G_FAILED;
-	
-        PRBool enumResult;
-        for (cookieEnumerator->HasMoreElements(&enumResult) ;
-             enumResult == PR_TRUE ;
-             cookieEnumerator->HasMoreElements(&enumResult))
-        {
-                CookieInfo *c;
-        
-                nsCOMPtr<nsICookie> nsCookie;
-                result = cookieEnumerator->GetNext (getter_AddRefs(nsCookie));
-                if (NS_FAILED(result)) return G_FAILED;
-
-                c = g_new0 (CookieInfo, 1);
-
-                nsCAutoString transfer;
-
-                nsCookie->GetHost (transfer);
-                c->domain = g_strdup (transfer.get());
-                nsCookie->GetName (transfer);
-                c->name = g_strdup (transfer.get());
-                nsCookie->GetValue (transfer);
-                c->value = g_strdup (transfer.get());
-                nsCookie->GetPath (transfer);
-                c->path = g_strdup (transfer.get());
-
-		PRBool isSecure;
-                nsCookie->GetIsSecure (&isSecure);
-                if (isSecure == PR_TRUE) 
-                        c->secure = g_strdup (_("Yes"));
-                else 
-                        c->secure = g_strdup (_("No"));
-
-                PRUint64 dateTime;
-                nsCookie->GetExpires (&dateTime);
-		if(dateTime == 0)
-			c->expire = g_strdup (_("End of current session"));
-		else
-	                c->expire = g_strdup_printf ("%s",ctime((time_t*)&dateTime));
-                
-                *cookies = g_list_prepend (*cookies, c);
-        }       
-
-	*cookies = g_list_reverse (*cookies);
-		
-	return G_OK;
-}
-
-static gresult           
-impl_remove_cookies (EphyEmbedSingle *shell,
-		     GList *cookies)
+static GList *
+impl_list_cookies (EphyCookieManager *manager)
 {
 	nsresult result;
-	GList *cl;
-        nsCOMPtr<nsICookieManager> cookieManager =
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+	GList *cookies = NULL;
 	
-        for (cl = g_list_first(cookies) ; cl != NULL ; 
-             cl = g_list_next (cl))
-        {
-                CookieInfo *c = (CookieInfo *)cl->data;
-
-                result = cookieManager->Remove (nsDependentCString(c->domain),
-                                                nsDependentCString(c->name),
-                                                nsDependentCString(c->path),
-                                                PR_FALSE);
-                if (NS_FAILED(result)) return G_FAILED;
-        };
-
-	return G_OK;
-}
+	nsCOMPtr<nsICookieManager> cookieManager = 
+			do_GetService (NS_COOKIEMANAGER_CONTRACTID);
+	nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
+	result = cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
+	if (NS_FAILED(result) || !cookieEnumerator) return NULL;
 	
-static gresult           
-impl_list_passwords (EphyEmbedSingle *shell,
-		     PasswordType type, 
-		     GList **passwords)
-{
-        nsresult result = NS_ERROR_FAILURE;
-
-        nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
-        nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
-        if (type == PASSWORD_PASSWORD)
-                result = passwordManager->GetEnumerator 
-                                (getter_AddRefs(passwordEnumerator));
-        else if (type == PASSWORD_REJECT)
-                result = passwordManager->GetRejectEnumerator 
-                                (getter_AddRefs(passwordEnumerator));
-        if (NS_FAILED(result)) return G_FAILED;      
-
-        PRBool enumResult;
-        for (passwordEnumerator->HasMoreElements(&enumResult) ;
-             enumResult == PR_TRUE ;
-             passwordEnumerator->HasMoreElements(&enumResult))
-        {
-                nsCOMPtr<nsIPassword> nsPassword;
-                result = passwordEnumerator->GetNext 
-                                        (getter_AddRefs(nsPassword));
-                if (NS_FAILED(result)) return G_FAILED;
-
-                PasswordInfo *p = g_new0 (PasswordInfo, 1);
-
-                nsCAutoString transfer;
-                nsPassword->GetHost (transfer);
-                p->host = g_strdup (transfer.get());
-
-                if (type == PASSWORD_PASSWORD)
-                {
-                        nsAutoString unicodeName;
-                        nsPassword->GetUser (unicodeName);
-                        p->username = g_strdup(NS_ConvertUCS2toUTF8(unicodeName).get());
-                }
-
-		*passwords = g_list_prepend (*passwords, p);
-        }       
-
-	*passwords = g_list_reverse (*passwords);
-
-	return G_OK;
-}
-
-static gresult           
-impl_remove_passwords (EphyEmbedSingle *shell,
-		       GList *passwords, 
-		       PasswordType type)
-{
-	nsresult result = NS_ERROR_FAILURE;
-        nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
-
-        for (GList *l = g_list_first(passwords) ; l !=NULL ; 
-             l = g_list_next(l))
-        {
-                PasswordInfo *p = (PasswordInfo *)l->data;
-                if (type == PASSWORD_PASSWORD)
-                {
-                        result = passwordManager->RemoveUser (NS_LITERAL_CSTRING(p->host),
-                                                              NS_ConvertUTF8toUCS2(nsDependentCString(p->username)));
-                }
-                else if (type == PASSWORD_REJECT)
-                {
-                        result = passwordManager->RemoveReject
-                                        (nsDependentCString(p->host));
-                };
-
-                if (NS_FAILED(result)) return G_FAILED;
-        };
-	
-        return G_OK;
-}
-
-static gresult 
-impl_show_file_picker (EphyEmbedSingle *shell,
-		       GtkWidget *parentWidget, 
-		       const char *title,
-		       const char *directory,
-		       const char *file, 
-		       FilePickerMode mode,
-                       char **ret_fullpath, 
-                       FileFormat *file_formats, 
-		       int *ret_file_format)
-{
-	char *expanded_directory = NULL;
-	gresult result;
-
-        GFilePicker *filePicker = new GFilePicker ();
-
-	if (directory != NULL)
+	PRBool enumResult;
+	for (cookieEnumerator->HasMoreElements(&enumResult) ;
+	     enumResult == PR_TRUE ;
+	     cookieEnumerator->HasMoreElements(&enumResult))
 	{
-	        expanded_directory = gnome_vfs_expand_initial_tilde (directory);
+		nsCOMPtr<nsICookie> keks;
+		result = cookieEnumerator->GetNext (getter_AddRefs(keks));
+		if (NS_FAILED (result) || !keks) continue;
+
+		EphyCookie *cookie = mozilla_cookie_to_ephy_cookie (keks);
+
+		cookies = g_list_prepend (cookies, cookie);
+	}       
+	
+	return cookies;
+}
+
+static void
+impl_remove_cookie (EphyCookieManager *manager,
+		    const EphyCookie *cookie)
+{
+	nsresult rv;
+	nsCOMPtr<nsICookieManager> cookieManager =
+		do_GetService (NS_COOKIEMANAGER_CONTRACTID, &rv);
+	if (NS_FAILED (rv) || !cookieManager) return;
+
+	cookieManager->Remove (nsDependentCString(cookie->domain),
+			       nsDependentCString(cookie->name),
+			       nsDependentCString(cookie->path),
+			       PR_FALSE /* block */);
+}
+
+static void
+impl_clear_cookies (EphyCookieManager *manager)
+{
+	nsresult rv;
+	nsCOMPtr<nsICookieManager> cookieManager =
+		do_GetService (NS_COOKIEMANAGER_CONTRACTID, &rv);
+	if (NS_SUCCEEDED (rv))
+	{
+		cookieManager->RemoveAll ();
+	}
+}
+	
+static GList *
+impl_list_passwords (EphyPasswordManager *manager)
+{
+	GList *passwords = NULL;
+
+	nsresult result;
+	nsCOMPtr<nsIPasswordManager> passwordManager =
+			do_GetService (NS_PASSWORDMANAGER_CONTRACTID);
+	if (!passwordManager) return NULL;
+
+	nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
+	result = passwordManager->GetEnumerator 
+				(getter_AddRefs(passwordEnumerator));
+	if (NS_FAILED(result) || !passwordEnumerator) return NULL;      
+
+	PRBool enumResult;
+	for (passwordEnumerator->HasMoreElements(&enumResult) ;
+	     enumResult == PR_TRUE ;
+	     passwordEnumerator->HasMoreElements(&enumResult))
+	{
+		nsCOMPtr<nsIPassword> nsPassword;
+		result = passwordEnumerator->GetNext 
+					(getter_AddRefs(nsPassword));
+		if (NS_FAILED(result) || !nsPassword) continue;
+
+		EphyPasswordInfo *p = g_new0 (EphyPasswordInfo, 1);
+
+		nsCAutoString transfer;
+		nsPassword->GetHost (transfer);
+		p->host = g_strdup (transfer.get());
+
+		nsAutoString unicodeName;
+		nsPassword->GetUser (unicodeName);
+		p->username = g_strdup(NS_ConvertUCS2toUTF8(unicodeName).get());
+
+		p->password = NULL;
+
+		passwords = g_list_prepend (passwords, p);
+	}       
+	
+	return passwords;
+}
+
+static void
+impl_remove_password (EphyPasswordManager *manager,
+		      EphyPasswordInfo *info)
+{
+        nsCOMPtr<nsIPasswordManager> pm =
+                        do_GetService (NS_PASSWORDMANAGER_CONTRACTID);
+	if (pm)
+	{
+		pm->RemoveUser (nsDependentCString(info->host),
+				NS_ConvertUTF8toUCS2(nsDependentCString(info->username)));
+	}
+}
+
+static const char *permission_type_string [] =
+{
+	"cookie",
+	"image",
+	"popup"
+};
+
+void
+impl_permission_manager_add (EphyPermissionManager *manager,
+			     const char *host,
+			     EphyPermissionType type,
+			     EphyPermission permission)
+{
+	/* can only set allow or deny */
+	g_return_if_fail (permission != EPHY_PERMISSION_DEFAULT);
+
+	nsresult result;
+        nsCOMPtr<nsIPermissionManager> pm
+		(do_GetService (NS_PERMISSIONMANAGER_CONTRACTID, &result));
+	if (NS_FAILED (result) || !pm) return;
+
+	nsCOMPtr<nsIURI> uri;
+        result = NS_NewURI(getter_AddRefs(uri), host);
+        if (NS_FAILED(result) || !uri) return;
+
+	gboolean allow = (permission == EPHY_PERMISSION_ALLOWED);
+
+	pm->Add (uri,
+#if MOZILLA_SNAPSHOT >= 10
+		 permission_type_string [type],
+#else
+		 type,
+#endif
+		 allow ? (PRUint32) nsIPermissionManager::ALLOW_ACTION :
+			 (PRUint32) nsIPermissionManager::DENY_ACTION);
+}
+
+void
+impl_permission_manager_remove (EphyPermissionManager *manager,
+				const char *host,
+				EphyPermissionType type)
+{
+	nsresult result;
+        nsCOMPtr<nsIPermissionManager> pm
+		(do_GetService (NS_PERMISSIONMANAGER_CONTRACTID, &result));
+	if (NS_SUCCEEDED (result))
+	{
+#if MOZILLA_SNAPSHOT >= 10
+		pm->Remove (nsDependentCString (host), permission_type_string [type]);
+#else
+		pm->Remove (nsDependentCString (host), type);
+#endif
+	}
+}
+
+void
+impl_permission_manager_clear (EphyPermissionManager *manager)
+{
+	nsresult result;
+        nsCOMPtr<nsIPermissionManager> pm
+		(do_GetService (NS_PERMISSIONMANAGER_CONTRACTID, &result));
+	if (NS_SUCCEEDED (result))
+	{
+		pm->RemoveAll ();
+	}
+}
+
+EphyPermission
+impl_permission_manager_test (EphyPermissionManager *manager,
+			      const char *host,
+			      EphyPermissionType type)
+{
+	nsresult result;
+        nsCOMPtr<nsIPermissionManager> pm
+		(do_GetService (NS_PERMISSIONMANAGER_CONTRACTID, &result));
+	if (NS_FAILED (result) || !pm) return EPHY_PERMISSION_DEFAULT;
+
+	nsCOMPtr<nsIURI> uri;
+        result = NS_NewURI(getter_AddRefs(uri), host);
+        if (NS_FAILED(result) || !uri) return EPHY_PERMISSION_DEFAULT;
+
+	PRUint32 action;
+#if MOZILLA_SNAPSHOT >= 10
+	result = pm->TestPermission (uri, permission_type_string [type], &action);
+#else
+	result = pm->TestPermission (uri, type, &action);
+#endif
+	if (NS_FAILED (result)) return EPHY_PERMISSION_DEFAULT;
+
+	EphyPermission permission;
+
+	switch (action)
+	{
+		case nsIPermissionManager::ALLOW_ACTION:
+			permission = EPHY_PERMISSION_ALLOWED;
+			break;
+		case nsIPermissionManager::DENY_ACTION:
+			permission = EPHY_PERMISSION_DENIED;
+			break;
+		case nsIPermissionManager::UNKNOWN_ACTION:
+		default:
+			permission = EPHY_PERMISSION_DEFAULT;
+			break;
 	}
 
-        /* make sure the directory exists, and use the home directory
-         * otherwise */
-        if (!expanded_directory ||
-            !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
-        {
-                g_free (expanded_directory);
-                expanded_directory = g_strdup (g_get_home_dir());
-        }
-
-        nsCOMPtr<nsILocalFile> dir = 
-                                do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-        dir->InitWithNativePath (nsDependentCString(expanded_directory));
-        g_free (expanded_directory);
-
-        filePicker->InitWithGtkWidget (parentWidget, title, mode);
-	if (file)
+	return permission;
+}
+
+GList *
+impl_permission_manager_list (EphyPermissionManager *manager,
+			      EphyPermissionType type)
+{
+	GList *list = NULL;
+
+	nsresult result;
+        nsCOMPtr<nsIPermissionManager> pm
+		(do_GetService (NS_PERMISSIONMANAGER_CONTRACTID, &result));
+	if (NS_FAILED (result) || !pm) return NULL;
+
+	nsCOMPtr<nsISimpleEnumerator> pe;
+	result = pm->GetEnumerator(getter_AddRefs(pe));
+	if (NS_FAILED(result) || !pe) return NULL;
+	
+	PRBool more;
+	for (pe->HasMoreElements (&more); more == PR_TRUE; pe->HasMoreElements (&more))
 	{
-	        filePicker->SetDefaultString (NS_ConvertUTF8toUCS2(file).get());
+		nsCOMPtr<nsIPermission> perm;
+		result = pe->GetNext(getter_AddRefs(perm));
+		if (NS_FAILED(result) || !perm) continue;
+
+#if MOZILLA_SNAPSHOT >= 10
+		nsCAutoString str;
+		result = perm->GetType(str);
+		if (NS_FAILED (result)) continue;
+
+		if (str.Equals(permission_type_string[type]))
+#else
+		PRUint32 num;
+		result = perm->GetType(&num);
+		if (NS_FAILED (result)) continue;
+
+		if ((PRUint32) num == (PRUint32) type)
+#endif
+		{
+			EphyPermissionInfo *info = 
+				mozilla_permission_to_ephy_permission (perm);
+
+			if (info != NULL)
+			{
+				list = g_list_prepend (list, info);
+			}
+		}
 	}
-        filePicker->SetDisplayDirectory (dir);
-        
-        PRInt16 retval;
-        filePicker->Show (&retval);
+
+	return list;
+}
+
+static void
+mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
+
+	object_class->finalize = mozilla_embed_single_finalize;
+
+	g_type_class_add_private (object_class, sizeof(MozillaEmbedSinglePrivate));
+}
 
-        nsCOMPtr<nsILocalFile> local_file;
-	filePicker->GetFile (getter_AddRefs(local_file));
-	nsCAutoString tempFullPathStr;
-	local_file->GetNativePath (tempFullPathStr);
-	*ret_fullpath = g_strdup (tempFullPathStr.get());
+static void
+ephy_embed_single_iface_init (EphyEmbedSingleClass *iface)
+{
+	iface->clear_cache = impl_clear_cache;
+	iface->set_offline_mode = impl_set_offline_mode;
+	iface->load_proxy_autoconf = impl_load_proxy_autoconf;
+	iface->get_font_list = impl_get_font_list;
+}
 
-        result = (retval == nsIFilePicker::returnCancel) ? G_FAILED : G_OK;
+static void
+ephy_cookie_manager_iface_init (EphyCookieManagerIFace *iface)
+{
+	iface->list = impl_list_cookies;
+	iface->remove = impl_remove_cookie;
+	iface->clear = impl_clear_cookies;
+}
 
-	delete filePicker;
+static void
+ephy_password_manager_iface_init (EphyPasswordManagerIFace *iface)
+{
+	iface->add = NULL; /* not implemented yet */
+	iface->remove = impl_remove_password;
+	iface->list = impl_list_passwords;
+}
 
-	return result;
+static void
+ephy_permission_manager_iface_init (EphyPermissionManagerIFace *iface)
+{
+	iface->add = impl_permission_manager_add;
+	iface->remove = impl_permission_manager_remove;
+	iface->clear = impl_permission_manager_clear;
+	iface->test = impl_permission_manager_test;
+	iface->list = impl_permission_manager_list;
 }
