@@ -17,7 +17,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ephy-embed-shell.h"
@@ -33,13 +33,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <stdlib.h>
-#include "nsCOMPtr.h"
-#include "nsIPrefService.h"
-#include "nsIServiceManager.h"
+#include <gdk/gdkx.h>
+#include <nsCOMPtr.h>
+#include <nsIPrefService.h>
+#include <nsIServiceManager.h>
+#include <libgnome/gnome-i18n.h>
 
 #define MOZILLA_PREF_NO_PROXIES "network.proxy.no_proxies_on"
+#define MIGRATE_PIXEL_SIZE
 
 static void
 mozilla_cache_size_notifier (GConfClient *client,
@@ -348,7 +351,7 @@ mozilla_proxy_autoconfig_notifier (GConfClient *client,
 
 	url = eel_gconf_get_string (entry->key);
 
-	if (url)
+	if (url && url[0] != '\0')
 	{
 		ephy_embed_single_load_proxy_autoconf (single, url);
 	}
@@ -379,25 +382,87 @@ add_notification_and_notify (GConfClient		*client,
 	mozilla_notifiers = g_list_append (mozilla_notifiers,
 			                   GUINT_TO_POINTER (cnxn_id));
 
-	entry = gconf_client_get_entry (client, key, NULL, TRUE, &error);
-	if (eel_gconf_handle_error (&error))
-	{
-		return;
-	}
-	g_return_if_fail (entry != NULL);
-
-	func (client, cnxn_id, entry, user_data);
-
-	gconf_entry_free (entry);
+	gconf_client_notify (client, key);
 }
 
+#ifdef MIGRATE_PIXEL_SIZE
+
+#define INT_ROUND(a) gint((a) + 0.5f)
+
+/**
+ *  This function gets the dpi in the same way that mozilla gets the dpi, 
+ *  this allows us to convert from pixels to points easily
+ */
+static gint
+mozilla_get_dpi ()
+{
+	GtkSettings* settings = gtk_settings_get_default ();
+	gint dpi = 0;
+ 
+	/* Use the gdk-xft-dpi setting if it is set */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (G_OBJECT (settings)),
+					  "gtk-xft-dpi"))
+	{
+		g_object_get (G_OBJECT (settings), "gtk-xft-dpi", &dpi, NULL);
+		if (dpi) return INT_ROUND (dpi / PANGO_SCALE);
+	}
+
+	/* Fall back to what xft thinks it is */
+	char *val = XGetDefault (GDK_DISPLAY (), "Xft", "dpi");
+	if (val)
+	{
+		char *e;
+		double d = strtod(val, &e);
+		if (e != val) return INT_ROUND (d);
+	}
+	
+	/* Fall back to calculating manually from the gdk screen settings */
+	float screenWidthIn = float (gdk_screen_width_mm()) / 25.4f;
+	return INT_ROUND (gdk_screen_width() / screenWidthIn);
+}
+
+static void
+mozilla_migrate_font_gconf_key (const char *pixel_key, const char *point_key)
+{
+	int size;
+
+	size = eel_gconf_get_integer (pixel_key);
+
+	if (size > 0)
+	{
+		/* Use doubles to get more accurate arithmetic */
+		double dpi   = (double)mozilla_get_dpi ();
+		double value = (double)eel_gconf_get_integer (pixel_key);
+		gint point = INT_ROUND ((value * 72) / dpi);
+
+		eel_gconf_set_integer (point_key, point);
+	}
+}
+
+#endif
+
 void 
-mozilla_notifiers_init(EphyEmbedSingle *single) 
+mozilla_notifiers_init (EphyEmbedSingle *single) 
 {
 	GConfClient *client = eel_gconf_client_get_global ();
 	guint i;
-	guint n_fonts_languages;
-	const FontsLanguageInfo *fonts_language;
+	const EphyFontsLanguageInfo *font_languages;
+	guint n_font_languages;
+
+	eel_gconf_monitor_add ("/apps/epiphany/web");
+	eel_gconf_monitor_add ("/system/proxy");
+	eel_gconf_monitor_add ("/system/http_proxy");
+
+#ifdef MIGRATE_PIXEL_SIZE
+	gboolean migrate_size;
+
+	migrate_size = (eel_gconf_get_integer (CONF_SCHEMA_VERSION)
+			< EPIPHANY_SCHEMA_VERSION);
+	if (migrate_size)
+	{
+		eel_gconf_set_integer (CONF_SCHEMA_VERSION, EPIPHANY_SCHEMA_VERSION);
+	}
+#endif
 
 	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
 	{
@@ -435,63 +500,123 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 	}
 
 	/* fonts notifiers */
-	n_fonts_languages = ephy_langs_get_n_font_languages ();
-	fonts_language = ephy_langs_get_font_languages ();
-	for (i = 0; i < n_fonts_languages; i++)
+	font_languages = ephy_font_languages ();
+	n_font_languages = ephy_font_n_languages ();
+
+	for (i=0; i < n_font_languages; i++)
 	{
+		const char *code = font_languages[i].code;
 		guint k;
-		char *types [] = { "serif", "sans-serif", "cursive", "fantasy", "monospace" };
+		char *types [] = { "variable", "monospace" };
 		char key[255];
 		char *info;
+#ifdef MIGRATE_PIXEL_SIZE
+		char old_key[255];
+#endif
 		
 		for (k = 0; k < G_N_ELEMENTS (types); k++)
 		{
-			info = g_strconcat (types[k], ".", fonts_language[i].code, NULL);
+			info = g_strconcat (types[k], ".", code, NULL);
 			
 			g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, 
-			 	    types[k], 
-                 	 	    fonts_language[i].code);
+			 	    types[k], code);
 			add_notification_and_notify (client, key,
 						     (GConfClientNotifyFunc)mozilla_font_notifier,
 						     info);
-			font_infos = g_list_append (font_infos, info);
+			font_infos = g_list_prepend (font_infos, info);
 		}
 
-		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, fonts_language[i].code);
-		info = g_strconcat ("minimum-size", ".", fonts_language[i].code, NULL);
-		add_notification_and_notify (client, key,
-					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					     info);
-		font_infos = g_list_append (font_infos, info);
+#ifdef MIGRATE_PIXEL_SIZE
+                if (migrate_size)
+                {
+			char *type;
 
-		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, fonts_language[i].code);
-		info = g_strconcat ("size.fixed", ".", fonts_language[i].code, NULL);
-		add_notification_and_notify (client, key,
-					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					     info);
-		font_infos = g_list_append (font_infos, info);
+			type = eel_gconf_get_string (CONF_RENDERING_FONT_TYPE_OLD);
+			if (type && (strcmp (type, "serif") == 0 ||
+				     strcmp (type, "sans-serif") == 0))
+			{
+				char *family;
 
-		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, fonts_language[i].code);
-		info = g_strconcat ("size.variable", ".", fonts_language[i].code, NULL);
+				g_snprintf (old_key, 255, "%s_%s_%s",
+					    CONF_RENDERING_FONT, type, code);
+				g_snprintf (key, 255, "%s_%s_%s",
+					    CONF_RENDERING_FONT, "variable", code);
+
+				family = eel_gconf_get_string (old_key);
+				if (family)
+				{
+					eel_gconf_set_string (key, family);
+					g_free (family);
+				}
+			}
+
+			g_free (type);
+		}
+#endif
+
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, code);
+		info = g_strconcat ("minimum-size", ".", code, NULL);
 		add_notification_and_notify (client, key,
 					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
 					     info);
-		font_infos = g_list_append (font_infos, info);		
+		font_infos = g_list_prepend (font_infos, info);
+
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_MIN_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
+
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, code);
+		info = g_strconcat ("size.fixed", ".", code, NULL);
+		add_notification_and_notify (client, key,
+					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
+					     info);
+		font_infos = g_list_prepend (font_infos, info);
+
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_FIXED_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
+
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, code);
+		info = g_strconcat ("size.variable", ".", code, NULL);
+		add_notification_and_notify (client, key,
+					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
+					     info);
+		font_infos = g_list_prepend (font_infos, info);
+
+#ifdef MIGRATE_PIXEL_SIZE
+		if (migrate_size)
+		{
+			g_snprintf (old_key, 255, "%s_%s",
+				    CONF_RENDERING_FONT_VAR_SIZE_OLD, code);
+			mozilla_migrate_font_gconf_key (old_key, key);
+		}
+#endif
 	}
 }
 
 void 
 mozilla_notifiers_free (void)
 {
-	GList *l;
-	
-	ephy_notification_remove (&mozilla_notifiers);
+	eel_gconf_monitor_remove ("/apps/epiphany/web");
+	eel_gconf_monitor_remove ("/system/proxy");
+	eel_gconf_monitor_remove ("/system/http_proxy");
 
-	for (l = font_infos; l != NULL; l = l->next)
-	{
-		g_free (l->data);
-	}
-	
+	g_list_foreach (mozilla_notifiers, 
+		        (GFunc)eel_gconf_notification_remove, 
+		        NULL);
+	g_list_free(mozilla_notifiers);
+
+	g_list_foreach (font_infos, (GFunc) g_free, NULL);
 	g_list_free (font_infos);
 }
 
@@ -617,12 +742,14 @@ get_system_language ()
 		char *lang = (char *)sys_langs->data;
 
 		/* FIXME this probably need to be smarter */
+		/* FIXME this can be up to 8 chars, not just 2 */
 		if (strcmp (lang, "C") != 0)
 		{
 			return g_strndup (lang, 2);
 		}
 	}
 
+	/* fallback to english */
 	return g_strdup ("en");
 }
 
@@ -632,43 +759,68 @@ mozilla_language_notifier(GConfClient *client,
 			  GConfEntry *entry,
 			  EphyEmbedSingle *single)
 {
-	GSList *languages, *l;
+	GSList *languages, *l, *ulist = NULL;
 	GString *result;
 
 	languages = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
-	if (languages == NULL) return;
 
 	result = g_string_new ("");
 
+	/* substitute the system language */
+	l = g_slist_find_custom (languages, "system", (GCompareFunc) strcmp);
+	if (l != NULL)
+	{
+		char *sys_lang;
+		int index;
+
+		index = g_slist_position (languages, l);
+		g_free (l->data);
+		languages = g_slist_delete_link (languages, l);
+
+		sys_lang = get_system_language ();
+
+		if (sys_lang)
+		{
+			char **s;
+			int i = 0;
+
+			s = g_strsplit (sys_lang, ",", -1);
+			while (s[i] != NULL)
+			{
+				languages = g_slist_insert (languages, g_strdup (s[i]), index);
+
+				index ++;
+				i++;
+			}
+
+			g_strfreev (s);
+		}		
+	}
+
+	/* now make a list of unique entries */
 	for (l = languages; l != NULL; l = l->next)
 	{
-		char *lang = (char *)l->data;
+		if (g_slist_find_custom (ulist, l->data, (GCompareFunc) strcmp) == NULL)
+		{
+			ulist = g_slist_prepend (ulist, l->data);
+		}
+	}
+	ulist = g_slist_reverse (ulist);
 
-		if (strcmp (lang, "system") == 0)
-		{
-			char *sys_lang;
-			
-			sys_lang = get_system_language ();
-			if (sys_lang)
-			{
-				g_string_append (result, sys_lang);
-				g_free (sys_lang);
-			}
-		}
-		else
-		{
-			g_string_append (result, (char *)l->data);
-		}
+	for (l = ulist; l != NULL; l = l->next)
+	{
+		g_string_append (result, (char *)l->data);
 
 		if (l->next) g_string_append (result, ",");
 	}
-	
+
 	mozilla_prefs_set_string ("intl.accept_languages", result->str);
 
 	g_string_free (result, TRUE);
-	
+
 	g_slist_foreach (languages, (GFunc) g_free, NULL);
 	g_slist_free (languages);
+	g_slist_free (ulist);
 }
 
 static void
