@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-prefs.h,v 1.20 2003/11/19 19:19:52 chpe Exp $
  */
 
 #ifndef EPHY_PREFS_H
@@ -21,20 +23,25 @@
 
 G_BEGIN_DECLS
 
+#define EPIPHANY_SCHEMA_VERSION 1
+#define CONF_SCHEMA_VERSION "/apps/epiphany/schema_version"
+
 /* General */
 #define CONF_GENERAL_HOMEPAGE "/apps/epiphany/general/homepage"
-#define CONF_ASK_DOWNLOAD_DEST "/apps/epiphany/general/ask_download_dest"
 #define CONF_ALWAYS_SHOW_TABS_BAR "/apps/epiphany/general/always_show_tabs_bar"
 #define CONF_WINDOWS_SHOW_TOOLBARS "/apps/epiphany/general/show_toolbars"
 #define CONF_WINDOWS_SHOW_BOOKMARKS_BAR "/apps/epiphany/general/show_bookmarks_bar"
 #define CONF_WINDOWS_SHOW_STATUSBAR "/apps/epiphany/general/show_statusbar"
 #define CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL "/apps/epiphany/general/middle_click_open_url"
+#define CONF_AUTO_OPEN_DOWNLOADS "/apps/epiphany/general/auto_open_downloads"
+#define CONF_DESKTOP_IS_HOME_DIR "/apps/nautilus/preferences/desktop_is_home_dir"
 
 /* Directories */
 #define CONF_STATE_SAVE_DIR           "/apps/epiphany/directories/save"
 #define CONF_STATE_SAVE_IMAGE_DIR     "/apps/epiphany/directories/saveimage"
 #define CONF_STATE_OPEN_DIR           "/apps/epiphany/directories/open"
-#define CONF_STATE_DOWNLOADING_DIR    "/apps/epiphany/directories/download"
+#define CONF_STATE_DOWNLOAD_DIR    "/apps/epiphany/directories/download"
+#define CONF_STATE_UPLOAD_DIR	      "/apps/epiphany/directories/upload"
 
 /* System prefs */
 #define CONF_DESKTOP_FTP_HANDLER "/desktop/gnome/url-handlers/ftp/command"
