@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2001 Matthew Mueller
- *            (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *	      (C) 2003 Marco Pesenti Gritti <mpeseng@tin.it>
+ *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003 Marco Pesenti Gritti <mpeseng@tin.it>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,14 +22,15 @@
 #include "ephy-state.h"
 #include "ephy-file-helpers.h"
 #include "ephy-node-db.h"
-#include "ephy-types.h"
 #include "ephy-node-common.h"
 
 #include <string.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkpaned.h>
+#include <gtk/gtkexpander.h>
 
-#define STATES_FILE "states.xml"
+#define EPHY_STATES_XML_FILE	"states.xml"
+#define EPHY_STATES_XML_ROOT    "ephy_states"
 #define EPHY_STATES_XML_VERSION "1.0"
 
 enum
@@ -41,96 +42,31 @@ enum
 	EPHY_NODE_STATE_PROP_POSITION_X = 6,
 	EPHY_NODE_STATE_PROP_POSITION_Y = 7,
 	EPHY_NODE_STATE_PROP_SIZE = 8,
-	EPHY_NODE_STATE_PROP_POSITION = 9
+	EPHY_NODE_STATE_PROP_POSITION = 9,
+	EPHY_NODE_STATE_PROP_EXPANDED = 10
 };
 
 static EphyNode *states = NULL;
 static EphyNodeDb *states_db = NULL;
 
-static void
-ephy_states_load (void)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-	char *xml_file;
-	char *tmp;
-
-	xml_file = g_build_filename (ephy_dot_dir (),
-                                     STATES_FILE,
-                                     NULL);
-
-	if (g_file_test (xml_file, G_FILE_TEST_EXISTS) == FALSE)
-		return;
-
-	doc = xmlParseFile (xml_file);
-	if (doc == NULL)
-	{
-		g_warning ("Failed to load state file.\n");
-		return;
-	}
-
-	root = xmlDocGetRootElement (doc);
-
-	tmp = xmlGetProp (root, "version");
-	if (tmp != NULL && strcmp (tmp, EPHY_STATES_XML_VERSION) == 0)
-	{
-		child = root->children;
-	}
-	else
-	{
-		g_warning ("Old version of states.xml discarded");
-		child = NULL;
-	}
-	g_free (tmp);
-
-	for (; child != NULL; child = child->next)
-	{
-		EphyNode *node;
-
-		node = ephy_node_new_from_xml (states_db, child);
-	}
-
-	xmlFreeDoc (doc);
-
-	g_free (xml_file);
-}
-
 static void
 ephy_states_save (void)
 {
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	GPtrArray *children;
-	int i;
 	char *xml_file;
 
-	if (states == NULL) return;
-
 	xml_file = g_build_filename (ephy_dot_dir (),
-                                     STATES_FILE,
+				     EPHY_STATES_XML_FILE,
                                      NULL);
 
-	/* save nodes to xml */
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
+	ephy_node_db_write_to_xml_safe
+		(states_db, xml_file,
+		 EPHY_STATES_XML_ROOT,
+		 EPHY_STATES_XML_VERSION,
+		 NULL, /* comment */
+		 states, 0,
+		 NULL);
 
-	root = xmlNewDocNode (doc, NULL, "ephy_states", NULL);
-	xmlSetProp (root, "version", EPHY_STATES_XML_VERSION);
-	xmlDocSetRootElement (doc, root);
-
-	children = ephy_node_get_children (states);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_save_to_xml (kid, root);
-	}
-	ephy_node_thaw (states);
-
-	ephy_file_save_xml (xml_file, doc);
-	g_free (xml_file);
+	g_free (xml_file);	
 }
 
 static EphyNode *
@@ -155,7 +91,6 @@ find_by_name (const char *name)
 			result = kid;
 		}
 	}
-	ephy_node_thaw (states);
 
 	return result;
 }
@@ -165,9 +100,19 @@ ensure_states (void)
 {
 	if (states == NULL)
 	{
+		char *xml_file;
+
+		xml_file = g_build_filename (ephy_dot_dir (),
+					     EPHY_STATES_XML_FILE,
+					     NULL);
+
 		states_db = ephy_node_db_new (EPHY_NODE_DB_STATES);
 		states = ephy_node_new_with_id (states_db, STATES_NODE_ID);
-		ephy_states_load ();
+		ephy_node_db_load_from_file (states_db, xml_file,
+					     EPHY_STATES_XML_ROOT,
+					     EPHY_STATES_XML_VERSION);
+	
+		g_free (xml_file);
 	}
 }
 
@@ -489,6 +434,59 @@ ephy_state_add_paned (GtkWidget *paned,
 			  G_CALLBACK (paned_size_allocate_cb), node);
 }
 
+static void
+sync_expander_cb (GtkExpander *expander,
+		  GParamSpec *pspec,
+		  EphyNode *node)
+{
+	GValue value = { 0, };
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, gtk_expander_get_expanded (expander));
+	ephy_node_set_property (node, EPHY_NODE_STATE_PROP_EXPANDED, &value);
+	g_value_unset (&value);
+}
+
+void 
+ephy_state_add_expander	(GtkWidget *expander,
+			 const char *name,
+			 gboolean default_state)
+{
+	EphyNode *node;
+	gboolean expanded;
+
+	ensure_states ();
+
+	node = find_by_name (name);
+	if (node == NULL)
+	{
+		GValue value = { 0, };
+
+		node = ephy_node_new (states_db);
+		ephy_node_add_child (states, node);
+
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_set_string (&value, name);
+		ephy_node_set_property (node, EPHY_NODE_STATE_PROP_NAME,
+				        &value);
+		g_value_unset (&value);
+
+		g_value_init (&value, G_TYPE_BOOLEAN);
+		g_value_set_boolean (&value, default_state);
+		ephy_node_set_property
+			(node, EPHY_NODE_STATE_PROP_EXPANDED, &value);
+		g_value_unset (&value);
+	}
+
+	expanded = ephy_node_get_property_boolean
+		(node, EPHY_NODE_STATE_PROP_EXPANDED);
+
+	gtk_expander_set_expanded (GTK_EXPANDER (expander), expanded);
+
+	g_signal_connect (expander, "notify::expanded",
+			  G_CALLBACK (sync_expander_cb), node);
+}
+
 void
 ephy_state_save (void)
 {
