@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.33 2003/10/14 11:36:37 marco Exp $
+ *  $Id: ephy-location-entry.c,v 1.35 2003/12/16 21:34:13 chpe Exp $
  */
 
 #include "ephy-tree-model-node.h"
@@ -242,6 +242,8 @@ completion_func (GtkEntryCompletion *completion,
 			if (!strncmp (key_prefixed, case_normalized_string,
 				      strlen (key_prefixed)))
 			{
+				g_free (key_prefixed);
+
 				ret = TRUE;
 				break;
 			}
@@ -253,6 +255,9 @@ completion_func (GtkEntryCompletion *completion,
 	g_free (item);
 	g_free (normalized_string);
 	g_free (case_normalized_string);
+	g_free (keywords);
+	g_free (normalized_keywords);
+	g_free (case_normalized_keywords);
 
 	return ret;
 }
