@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-zoom-action.c,v 1.4 2003/11/05 22:56:12 chpe Exp $
+ *  $Id: ephy-zoom-action.c,v 1.5 2003/12/15 14:06:53 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -108,7 +108,8 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 	g_signal_connect_object (action, "notify::zoom",
 				 G_CALLBACK (sync_zoom_cb), proxy, 0);
 
-	g_signal_connect (proxy, "zoom_to_level", GTK_SIGNAL_FUNC(zoom_to_level_cb), action);
+	g_signal_connect (proxy, "zoom_to_level",
+			  G_CALLBACK (zoom_to_level_cb), action);
 
 	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
 }
