@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-encoding-dialog.c,v 1.7 2003/11/30 14:08:32 chpe Exp $
+ *  $Id: ephy-encoding-dialog.c,v 1.9 2003/12/19 20:16:51 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -107,14 +107,6 @@ ephy_encoding_dialog_get_type (void)
 	return ephy_type_encoding_dialog;
 }
 
-static gboolean
-encoding_is_automatic (EphyEncodingInfo *info)
-{
-	g_return_val_if_fail (info != NULL, FALSE);
-	
-	return (info->encoding_source < EMBED_ENCODING_PARENT_FORCED);
-}
-
 static void
 sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 {
@@ -136,7 +128,7 @@ sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 	if (info == NULL) return;
 
 	node = ephy_encodings_get_node (dialog->priv->encodings, info->encoding);
-	g_return_if_fail (EPHY_IS_NODE (node));
+	if (!EPHY_IS_NODE (node)) return;
 
 	/* select the current encoding in the list view */
 	ephy_node_view_select_node (EPHY_NODE_VIEW (dialog->priv->enc_view),
@@ -159,7 +151,7 @@ sync_embed_cb (EphyEncodingDialog *dialog, GParamSpec *pspec, gpointer dummy)
 		g_list_free (rows);
 	}
 
-	is_automatic = encoding_is_automatic (info);
+	is_automatic = ephy_encoding_info_is_automatic (info);
 
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), is_automatic);
