@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +15,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-tab.c,v 1.76 2003/12/19 14:35:55 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,17 +27,17 @@
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-embed-factory.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-debug.h"
-#include "egg-menu-merge.h"
 #include "ephy-string.h"
 #include "ephy-notebook.h"
 #include "ephy-file-helpers.h"
 #include "ephy-zoom.h"
-#include "session.h"
 #include "ephy-favicon-cache.h"
+#include "ephy-embed-persist.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkbutton.h>
@@ -47,8 +50,12 @@
 #include <gtk/gtkselection.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenu.h>
+#include <gtk/gtkuimanager.h>
+#include <gtk/gtkradioaction.h>
 #include <string.h>
 
+#define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
+
 struct EphyTabPrivate
 {
 	EphyEmbed *embed;
@@ -66,8 +73,9 @@ struct EphyTabPrivate
 	int total_requests;
 	int width;
 	int height;
-	EggAction *action;
+	GtkToggleAction *action;
 	float zoom;
+	gboolean setting_zoom;
 	EmbedSecurityLevel security_level;
 	TabNavigationFlags nav_flags;
 };
@@ -102,7 +110,7 @@ static void	ephy_tab_set_icon_address	(EphyTab *tab,
 static void	ephy_tab_set_load_status	(EphyTab *tab,
 						 gboolean status);
 static void	ephy_tab_set_link_message	(EphyTab *tab,
-						 const char *message);
+						 char *message);
 static void	ephy_tab_set_load_percent	(EphyTab *tab,
 						 int percent);
 static void	ephy_tab_update_navigation_flags(EphyTab *tab);
@@ -136,8 +144,8 @@ ephy_tab_get_type (void)
                 };
 
                 ephy_tab_type = g_type_register_static (G_TYPE_OBJECT,
-							  "EphyTab",
-							  &our_info, 0);
+							"EphyTab",
+							&our_info, 0);
         }
 
         return ephy_tab_type;
@@ -222,11 +230,15 @@ ephy_tab_get_property (GObject *object,
 }
 
 static void
-ephy_tab_action_activate_cb (EggAction *action, EphyTab *tab)
+ephy_tab_action_activate_cb (GtkAction *action, EphyTab *tab)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	ephy_window_jump_to_tab (tab->priv->window, tab);
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) &&
+	    ephy_window_get_active_tab (tab->priv->window) != tab)
+	{
+		ephy_window_jump_to_tab (tab->priv->window, tab);
+	}
 }
 
 static void
@@ -325,7 +337,7 @@ ephy_tab_class_init (EphyTabClass *class)
 					 g_param_spec_object ("window",
 							      "Window",
 							      "The tab's parent window",
-							      EPHY_WINDOW_TYPE,
+							      EPHY_TYPE_WINDOW,
 							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -337,6 +349,8 @@ ephy_tab_class_init (EphyTabClass *class)
 							     ZOOM_MAXIMAL,
 							     1.0,
 							     G_PARAM_READABLE));
+
+	g_type_class_add_private (object_class, sizeof(EphyTabPrivate));
 }
 
 static void
@@ -360,13 +374,7 @@ ephy_tab_embed_destroy_cb (GtkWidget *widget, EphyTab *tab)
 static void
 ephy_tab_finalize (GObject *object)
 {
-        EphyTab *tab;
-
-	g_return_if_fail (IS_EPHY_TAB (object));
-
-	tab = EPHY_TAB (object);
-
-        g_return_if_fail (tab->priv != NULL);
+        EphyTab *tab = EPHY_TAB (object);
 
 	g_idle_remove_by_data (tab->priv->embed);
 
@@ -381,13 +389,34 @@ ephy_tab_finalize (GObject *object)
 	g_free (tab->priv->link_message);
 	g_free (tab->priv->status_message);
 
-        g_free (tab->priv);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
 	LOG ("EphyTab finalized %p", tab)
 }
 
+static gboolean
+address_has_web_scheme (const char *address)
+{
+	GnomeVFSURI *uri;
+	const char *scheme;
+	gboolean has_web_scheme = FALSE;
+
+	uri = gnome_vfs_uri_new (address);
+	if (uri != NULL)
+	{
+		scheme = gnome_vfs_uri_get_scheme (uri);
+
+		has_web_scheme = (strcmp (scheme, "http") == 0 ||
+				  strcmp (scheme, "https") == 0 ||
+				  strcmp (scheme, "ftp") == 0 ||
+				  strcmp (scheme, "file") == 0);
+
+		gnome_vfs_uri_unref (uri);
+	}
+
+	return has_web_scheme;
+}
+
 /* Public functions */
 
 EphyTab *
@@ -395,15 +424,15 @@ ephy_tab_new (void)
 {
 	EphyTab *tab;
 
-	tab = EPHY_TAB (g_object_new (EPHY_TAB_TYPE, NULL));
-	g_return_val_if_fail (tab->priv != NULL, NULL);
+	tab = EPHY_TAB (g_object_new (EPHY_TYPE_TAB, NULL));
+
 	return tab;
 }
 
 static void
 ephy_tab_set_load_status (EphyTab *tab, gboolean status)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	tab->priv->load_status = status;
 
@@ -413,18 +442,18 @@ ephy_tab_set_load_status (EphyTab *tab, gboolean status)
 gboolean
 ephy_tab_get_load_status (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), FALSE);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
 
 	return tab->priv->load_status;
 }
 
 static void
-ephy_tab_set_link_message (EphyTab *tab, const char *message)
+ephy_tab_set_link_message (EphyTab *tab, char *message)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_free (tab->priv->link_message);
-	tab->priv->link_message = g_strdup (message);
+	tab->priv->link_message = ephy_string_blank_chr (message);
 
 	g_object_notify (G_OBJECT (tab), "message");
 }
@@ -432,7 +461,7 @@ ephy_tab_set_link_message (EphyTab *tab, const char *message)
 const char *
 ephy_tab_get_link_message (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), NULL);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab->priv->link_message;
 }
@@ -440,7 +469,7 @@ ephy_tab_get_link_message (EphyTab *tab)
 EphyEmbed *
 ephy_tab_get_embed (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), NULL);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab->priv->embed;
 }
@@ -448,8 +477,8 @@ ephy_tab_get_embed (EphyTab *tab)
 void
 ephy_tab_set_window (EphyTab *tab, EphyWindow *window)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
-	if (window) g_return_if_fail (IS_EPHY_WINDOW (window));
+	g_return_if_fail (EPHY_IS_TAB (tab));
+	if (window) g_return_if_fail (EPHY_IS_WINDOW (window));
 
 	if (window != tab->priv->window)
 	{
@@ -462,7 +491,7 @@ ephy_tab_set_window (EphyTab *tab, EphyWindow *window)
 EphyWindow *
 ephy_tab_get_window (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), NULL);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab->priv->window;
 }
@@ -470,7 +499,7 @@ ephy_tab_get_window (EphyTab *tab)
 void
 ephy_tab_get_size (EphyTab *tab, int *width, int *height)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	*width = tab->priv->width;
 	*height = tab->priv->height;
@@ -479,7 +508,7 @@ ephy_tab_get_size (EphyTab *tab, int *width, int *height)
 gboolean
 ephy_tab_get_visibility (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), FALSE);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
 
 	return tab->priv->visibility;
 }
@@ -488,7 +517,7 @@ void
 ephy_tab_set_visibility (EphyTab *tab,
                          gboolean visible)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	tab->priv->visibility = visible;
 
@@ -517,7 +546,7 @@ ephy_tab_set_icon_address (EphyTab *tab, const char *address)
 	EphyBookmarks *eb;
 	EphyHistory *history;
 
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_free (tab->priv->icon_address);
 
@@ -540,7 +569,7 @@ ephy_tab_set_icon_address (EphyTab *tab, const char *address)
 const char *
 ephy_tab_get_icon_address (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), NULL);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return tab->priv->icon_address;
 }
@@ -557,15 +586,16 @@ ephy_tab_favicon_cb (EphyEmbed *embed,
 
 static void
 ephy_tab_link_message_cb (EphyEmbed *embed,
-			  const char *message,
 			  EphyTab *tab)
 {
-	ephy_tab_set_link_message (tab, message);
+	ephy_tab_set_link_message (tab, ephy_embed_get_link_message (embed));
 }
 
 static void
 ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
+	LOG ("ephy_tab_address_cb tab %p address %s", tab, address)
+
 	if (tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
 	{
 		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
@@ -574,12 +604,75 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
 	ephy_tab_update_navigation_flags (tab);
+
+	/* restore zoom level */
+	if (address_has_web_scheme (address))
+	{
+		EphyHistory *history;
+		EphyNode *host;
+		GValue value = { 0, };
+		float zoom = 1.0, current_zoom;
+
+		history = ephy_embed_shell_get_global_history
+				(EPHY_EMBED_SHELL (ephy_shell));
+		host = ephy_history_get_host (history, address);
+
+		if (host != NULL && ephy_node_get_property
+				     (host, EPHY_NODE_HOST_PROP_ZOOM, &value))
+		{
+			zoom = g_value_get_float (&value);
+			g_value_unset (&value);
+		}
+
+		current_zoom = ephy_embed_zoom_get (embed);
+		if (zoom != current_zoom)
+		{
+			tab->priv->setting_zoom = TRUE;
+			ephy_embed_zoom_set (embed, zoom, FALSE);
+			tab->priv->setting_zoom = FALSE;
+		}
+	}
 }
 
 static void
 ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 {
+	char *address;
+
+	LOG ("ephy_tab_zoom_changed_cb tab %p zoom %f", tab, zoom)
+
 	ephy_tab_set_zoom (tab, zoom);
+
+	if (tab->priv->setting_zoom)
+	{
+		return;
+	}
+
+	address = ephy_embed_get_location (embed, TRUE);
+	if (address_has_web_scheme (address))
+	{
+		EphyHistory *history;
+		EphyNode *host;
+		GValue value = { 0, };
+		history = ephy_embed_shell_get_global_history
+				(EPHY_EMBED_SHELL (ephy_shell));
+		host = ephy_history_get_host (history, address);
+
+		if (host != NULL)
+		{
+			float zoom;
+
+			zoom = ephy_embed_zoom_get (embed);
+
+			g_value_init (&value, G_TYPE_FLOAT);
+			g_value_set_float (&value, zoom);
+			ephy_node_set_property
+				(host, EPHY_NODE_HOST_PROP_ZOOM, &value);
+			g_value_unset (&value);
+		}
+	}
+
+	g_free (address);
 }
 
 static void
@@ -587,7 +680,7 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 {
 	char *title;
 
-	ephy_embed_get_title (embed, &title);
+	title = ephy_embed_get_title (embed);
 
 	ephy_tab_set_title (tab, title);
 
@@ -595,16 +688,24 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 }
 
 static int
-build_load_percent (int bytes_loaded, int max_bytes_loaded)
+build_load_percent (int requests_done, int requests_total)
 {
-	if (max_bytes_loaded > 0)
+	int percent;
+
+	if (requests_total > 0)
 	{
-		return (bytes_loaded * 100) / max_bytes_loaded;
+		percent = (requests_done * 100) / requests_total;
+
+		/* Mozilla sometimes report more done requests than
+		   total requests. Their progress widget clamp the value */
+		percent = CLAMP (percent, 0, 100);
 	}
 	else
 	{
-		return -1;
+		percent = -1;
 	}
+
+	return percent;
 }
 
 static char *
@@ -636,11 +737,11 @@ get_host_name_from_uri (const char *uri)
 	return result;
 }
 
-static char *
-build_net_state_message (const char *uri, EmbedState flags)
+static void
+update_net_state_message (EphyTab *tab, const char *uri, EmbedState flags)
 {
 	const char *msg = NULL;
-	char *host, *message = NULL;
+	char *host;
 
 	host = get_host_name_from_uri (uri);
 
@@ -668,20 +769,24 @@ build_net_state_message (const char *uri, EmbedState flags)
                 {
                         msg = _("Loading %s...");
                 }
-                else if (flags & EMBED_STATE_STOP)
-                {
-			msg = _("Done.");
-                }
         }
 
-	if (msg)
+	if ((flags & EMBED_STATE_IS_NETWORK) &&
+	    (flags & EMBED_STATE_STOP))
+        {
+		g_free (tab->priv->status_message);
+		tab->priv->status_message = NULL;
+		g_object_notify (G_OBJECT (tab), "message");
+
+	}
+	else if (msg)
 	{
-		message = g_strdup_printf (msg, host); 
+		g_free (tab->priv->status_message);
+		tab->priv->status_message = g_strdup_printf (msg, host);
+		g_object_notify (G_OBJECT (tab), "message");
 	}
 
 	g_free (host);
-
-	return message;
 }
 
 static void
@@ -702,10 +807,8 @@ build_progress_from_requests (EphyTab *tab, EmbedState state)
 
 		load_percent = build_load_percent (tab->priv->cur_requests,
 						   tab->priv->total_requests);
-		if (load_percent > tab->priv->load_percent || load_percent == -1)
-		{
-			ephy_tab_set_load_percent (tab, load_percent);
-		}
+
+		ephy_tab_set_load_percent (tab, load_percent);
 	}
 }
 
@@ -728,16 +831,7 @@ static void
 ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		       EmbedState state, EphyTab *tab)
 {
-	char *new_msg;
-
-	new_msg = build_net_state_message (uri, state);
-	if (new_msg)
-	{
-		g_free (tab->priv->status_message);
-		tab->priv->status_message = new_msg;
-	}
-
-	g_object_notify (G_OBJECT (tab), "message");
+	update_net_state_message (tab, uri, state);
 
 	if (state & EMBED_STATE_IS_NETWORK)
 	{
@@ -753,12 +847,6 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		}
 		else if (state & EMBED_STATE_STOP)
 		{
-			/* tab load completed, save in session */
-			Session *s;
-
-			s = SESSION (ephy_shell_get_session (ephy_shell));
-			session_save (s, SESSION_CRASHED);
-
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, FALSE);
 			ephy_tab_update_navigation_flags (tab);
@@ -777,7 +865,7 @@ ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
 	EphyWindow *window;
 
 	window = ephy_window_new ();
-	ephy_window_set_chrome (window, chromemask);
+	ephy_window_request_chrome (window, chromemask);
 
 	new_tab = ephy_tab_new ();
         ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
@@ -812,44 +900,33 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 static void
 ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 	g_return_if_fail (tab->priv->window != NULL);
 
 	ephy_window_remove_tab (tab->priv->window, tab);
 }
 
-static gint
-ephy_tab_open_uri_cb (EphyEmbed *embed, const char *uri,
-			EphyTab *tab)
-{
-	LOG ("ephy_tab_open_uri_cb %s", uri)
-
-	/* FIXME: what is this function supposed to do ? */
-	return FALSE;
-}
-
 static void
 ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 		     EphyTab *tab)
 {
-	GList *tabs = NULL;
+	GtkWidget *notebook;
 	EphyWindow *window;
 	GtkWidget *widget;
-	EmbedChromeMask chromemask;
 
 	tab->priv->width = width;
 	tab->priv->height = height;
 
 	window = tab->priv->window;
-	tabs = ephy_window_get_tabs (window);
+	notebook = ephy_window_get_notebook (window);
 	widget = GTK_WIDGET (embed);
-	chromemask = ephy_window_get_chrome (window);
 
 	/* Do not resize window with multiple tabs.
 	 * Do not resize window already showed because
 	 * it's not possible to calculate a sensible window
 	 * size based on the embed size */
-	if (g_list_length (tabs) == 1 && !tab->priv->visibility)
+	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) == 1 &&
+	    !tab->priv->visibility)
 	{
 		gtk_widget_set_size_request
 			(widget, width, height);
@@ -858,59 +935,141 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 		 * has been resized. It appears to be the only way
 		 * to have the window sized according to embed
 		 * size correctly.
-		 * We dont do it for XUL dialogs because in that case
-		 * a "forced" requisition appear correct.
 		 */
-		if (!(chromemask & EMBED_CHROME_OPENASCHROME))
-		{
-			g_idle_add (let_me_resize_hack, embed);
-		}
+		g_idle_add (let_me_resize_hack, embed);
 	}
-
-	g_list_free (tabs);
 }
 
-static gint
-ephy_tab_dom_mouse_click_cb  (EphyEmbed *embed,
-			      EphyEmbedEvent *event,
-			      EphyTab *tab)
+static void
+open_link_in_new_tab (EphyTab *tab,
+		      const char *link_address)
 {
 	EphyWindow *window;
+	gboolean new_tab;
+
+	window = ephy_tab_get_window (tab);
+	g_return_if_fail (window != NULL);
+
+	new_tab = address_has_web_scheme (link_address);
+
+	if (new_tab)
+	{
+		ephy_shell_new_tab (ephy_shell, window, tab,
+				    link_address,
+				    EPHY_NEW_TAB_OPEN_PAGE |
+				    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+	}
+	else
+	{
+		ephy_window_load_url (window, link_address);
+	}
+}
+
+static void
+save_property_url (EphyEmbed *embed,
+		   EphyEmbedEvent *event,
+		   const char *property,
+		   const char *key)
+{
+	const char *location;
+	const GValue *value;
+	EphyEmbedPersist *persist;
+
+	ephy_embed_event_get_property (event, property, &value);
+	location = g_value_get_string (value);
+
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_flags (persist, 0);
+	ephy_embed_persist_set_persist_key (persist, key);
+	ephy_embed_persist_set_source (persist, location);
+
+	ephy_embed_persist_save (persist);
+
+	g_object_unref (G_OBJECT(persist));
+}
+
+static gboolean
+ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
+			     EphyEmbedEvent *event,
+			     EphyTab *tab)
+{
 	EphyEmbedEventType type;
 	EmbedEventContext context;
+	EphyWindow *window;
+	guint modifier;
+	gboolean handled = TRUE;
+	gboolean with_control, with_shift, is_left_click, is_middle_click;
+	gboolean is_link, is_image, is_middle_clickable;
+	gboolean middle_click_opens;
+	gboolean is_input;
 
-	g_assert (IS_EPHY_EMBED_EVENT(event));
+	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
 	window = ephy_tab_get_window (tab);
 	g_return_val_if_fail (window != NULL, FALSE);
 
-	ephy_embed_event_get_event_type (event, &type);
-	ephy_embed_event_get_context (event, &context);
+	type = ephy_embed_event_get_event_type (event);
+	context = ephy_embed_event_get_context (event);
+	modifier = ephy_embed_event_get_modifier (event);
 
-	if (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2
-	    && (context & EMBED_CONTEXT_LINK))
+	LOG ("ephy_tab_dom_mouse_click_cb: type %d, context %x, modifier %x",
+	     type, context, modifier)
+
+	with_control = (modifier & GDK_CONTROL_MASK) != 0;
+	with_shift = (modifier & GDK_SHIFT_MASK) != 0;
+	is_left_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON1);
+	is_middle_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2);
+
+	middle_click_opens = eel_gconf_get_boolean
+				(CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL);
+
+	is_link = (context & EMBED_CONTEXT_LINK) != 0;
+	is_image = (context & EMBED_CONTEXT_IMAGE) != 0;
+	is_middle_clickable = !((context & EMBED_CONTEXT_LINK)
+				|| (context & EMBED_CONTEXT_INPUT)
+				|| (context & EMBED_CONTEXT_EMAIL_LINK));
+	is_input = (context & EMBED_CONTEXT_INPUT) != 0;
+
+	/* ctrl+click or middle click opens the link in new tab */
+	if (is_link && ((is_left_click && with_control) || is_middle_click))
 	{
 		const GValue *value;
+		const char *link_address;
 
 		ephy_embed_event_get_property (event, "link", &value);
-		ephy_shell_new_tab (ephy_shell, window, tab,
-				      g_value_get_string (value),
-				      EPHY_NEW_TAB_OPEN_PAGE);
+		link_address = g_value_get_string (value);
+		open_link_in_new_tab (tab, link_address);
 	}
-	else if (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2 &&
-		 eel_gconf_get_boolean (CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL) &&
-		 !(context & EMBED_CONTEXT_LINK
-		   || context & EMBED_CONTEXT_EMAIL_LINK
-		   || context & EMBED_CONTEXT_INPUT))
+	/* shift+click saves the link target */
+	else if (is_link && is_left_click && with_shift)
+	{
+		save_property_url (embed, event, "link", CONF_STATE_DOWNLOAD_DIR);
+	}
+	/* shift+click saves the non-link image
+	 * Note: pressing enter to submit a form synthesizes a mouse click event
+	 */
+	else if (is_image && is_left_click && with_shift && !is_input)
+	{
+		save_property_url (embed, event, "image", CONF_STATE_SAVE_IMAGE_DIR);
+	}
+	/* middle click opens the selection url */
+	else if (is_middle_clickable && is_middle_click && middle_click_opens)
 	{
-		/* paste url */
 		gtk_selection_convert (GTK_WIDGET (window),
 				       GDK_SELECTION_PRIMARY,
 				       GDK_SELECTION_TYPE_STRING,
 				       GDK_CURRENT_TIME);
 	}
+	/* we didn't handle the event */
+	else
+	{
+		handled = FALSE;
+	}
 
-	return FALSE;
+	return handled;
 }
 
 static void
@@ -924,16 +1083,12 @@ static void
 ephy_tab_init (EphyTab *tab)
 {
 	GObject *embed, *embed_widget;
-	EphyEmbedSingle *single;
 	EphyFaviconCache *cache;
 	char *id;
 
 	LOG ("EphyTab initialising %p", tab)
 
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
-
-        tab->priv = g_new0 (EphyTabPrivate, 1);
+	tab->priv = EPHY_TAB_GET_PRIVATE (tab);
 
 	tab->priv->window = NULL;
 	tab->priv->status_message = NULL;
@@ -949,21 +1104,25 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->load_status = FALSE;
 	tab->priv->link_message = NULL;
 	tab->priv->security_level = STATE_IS_UNKNOWN;
-	tab->priv->status_message = NULL;
 	tab->priv->zoom = 1.0;
+	tab->priv->setting_zoom = FALSE;
 	tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 
-	tab->priv->embed = ephy_embed_new (G_OBJECT(single));
+	tab->priv->embed = EPHY_EMBED
+		(ephy_embed_factory_new_object ("EphyEmbed"));
+	g_assert (tab->priv->embed != NULL);
 
 	embed = G_OBJECT (tab->priv->embed);
 	embed_widget = G_OBJECT (tab->priv->embed);
 
 	id = g_strdup_printf ("Tab%lu", tab_id++);
 
-	tab->priv->action = g_object_new (EGG_TYPE_ACTION,
+	tab->priv->action = g_object_new (GTK_TYPE_TOGGLE_ACTION,
 					  "name", id,
 					  "label", _("Blank page"),
+					  "draw_as_radio", TRUE,
 					  NULL);
+
 	g_free (id);
 	g_signal_connect (tab->priv->action, "activate",
 			  G_CALLBACK (ephy_tab_action_activate_cb), tab);
@@ -977,13 +1136,13 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (embed_widget, "destroy",
 			  G_CALLBACK (ephy_tab_embed_destroy_cb),
 			  tab);
-	g_signal_connect (embed, "ge_link_message",
+	g_signal_connect (embed, "link_message",
 			  G_CALLBACK (ephy_tab_link_message_cb),
 			  tab);
 	g_signal_connect (embed, "ge_location",
 			  G_CALLBACK (ephy_tab_address_cb),
 			  tab);
-	g_signal_connect (embed, "ge_title",
+	g_signal_connect (embed, "title",
 			  G_CALLBACK (ephy_tab_title_cb),
 			  tab);
 	g_signal_connect (embed, "ge_zoom_change",
@@ -995,16 +1154,13 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (embed, "ge_new_window",
 			  G_CALLBACK (ephy_tab_new_window_cb),
 			  tab);
-	g_signal_connect (embed, "ge_visibility",
+	g_signal_connect (embed, "visibility",
 			  G_CALLBACK (ephy_tab_visibility_cb),
 			  tab);
-	g_signal_connect (embed, "ge_destroy_brsr",
+	g_signal_connect (embed, "destroy_browser",
 			  G_CALLBACK (ephy_tab_destroy_brsr_cb),
 			  tab);
-	g_signal_connect (embed, "ge_open_uri",
-			  G_CALLBACK (ephy_tab_open_uri_cb),
-			  tab);
-	g_signal_connect (embed, "ge_size_to",
+	g_signal_connect (embed, "size_to",
 			  G_CALLBACK (ephy_tab_size_to_cb),
 			  tab);
 	g_signal_connect (embed, "ge_dom_mouse_click",
@@ -1027,7 +1183,7 @@ ephy_tab_init (EphyTab *tab)
 void
 ephy_tab_set_load_percent (EphyTab *tab, int percent)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	if (percent != tab->priv->load_percent)
 	{
@@ -1040,7 +1196,7 @@ ephy_tab_set_load_percent (EphyTab *tab, int percent)
 int
 ephy_tab_get_load_percent (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), -1);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), -1);
 
 	return tab->priv->load_percent;
 }
@@ -1053,17 +1209,17 @@ ephy_tab_update_navigation_flags (EphyTab *tab)
 
 	embed = tab->priv->embed;
 
-	if (ephy_embed_can_go_up (embed) == G_OK)
+	if (ephy_embed_can_go_up (embed))
 	{
 		flags |= TAB_NAV_UP;
 	}
 
-	if (ephy_embed_can_go_back (embed) == G_OK)
+	if (ephy_embed_can_go_back (embed))
 	{
 		flags |= TAB_NAV_BACK;
 	}
 
-	if (ephy_embed_can_go_forward (embed) == G_OK)
+	if (ephy_embed_can_go_forward (embed))
 	{
 		flags |= TAB_NAV_FORWARD;
 	}
@@ -1079,7 +1235,7 @@ ephy_tab_update_navigation_flags (EphyTab *tab)
 TabNavigationFlags
 ephy_tab_get_navigation_flags (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), 0);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), 0);
 
 	return tab->priv->nav_flags;
 }
@@ -1087,7 +1243,7 @@ ephy_tab_get_navigation_flags (EphyTab *tab)
 const char *
 ephy_tab_get_status_message (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), "");
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	if (tab->priv->link_message && tab->priv->link_message[0] != '\0')
 	{
@@ -1099,7 +1255,7 @@ ephy_tab_get_status_message (EphyTab *tab)
 	}
 	else
 	{
-		return "";
+		return NULL;
 	}
 }
 
@@ -1111,7 +1267,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 	char *title_short = NULL;
 	char *title = NULL;
 
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	g_free (tab->priv->title);
 
@@ -1120,7 +1276,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 		GnomeVFSURI *uri = NULL;
 		char *address;
 
-		ephy_embed_get_location (tab->priv->embed, TRUE, &address);
+		address = ephy_embed_get_location (tab->priv->embed, TRUE);
 
 		if (address)
 		{
@@ -1137,7 +1293,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 					GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
 			gnome_vfs_uri_unref (uri);
 		}
-		else if (address != NULL)
+		else if (address != NULL && strncmp (address, "about:blank", 11) != 0)
 		{
 			title = g_strdup (address);
 		}
@@ -1148,7 +1304,6 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 			title = g_strdup (_("Blank page"));
 		}
 
-
 		g_free (address);
 	}
 	else
@@ -1176,7 +1331,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 const char *
 ephy_tab_get_title (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), "");
+	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
 
 	return tab->priv->title;
 }
@@ -1184,7 +1339,7 @@ ephy_tab_get_title (EphyTab *tab)
 const char *
 ephy_tab_get_location (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), "");
+	g_return_val_if_fail (EPHY_IS_TAB (tab), "");
 
 	return tab->priv->address;
 }
@@ -1194,7 +1349,7 @@ ephy_tab_set_location (EphyTab *tab,
 		       const char *address,
 		       TabAddressExpire expire)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	if (tab->priv->address) g_free (tab->priv->address);
 	tab->priv->address = g_strdup (address);
@@ -1215,7 +1370,7 @@ ephy_tab_set_location (EphyTab *tab,
 static void
 ephy_tab_set_security_level (EphyTab *tab, EmbedSecurityLevel level)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	tab->priv->security_level = level;
 
@@ -1225,7 +1380,7 @@ ephy_tab_set_security_level (EphyTab *tab, EmbedSecurityLevel level)
 EmbedSecurityLevel
 ephy_tab_get_security_level (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), STATE_IS_UNKNOWN);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), STATE_IS_UNKNOWN);
 
 	return tab->priv->security_level;
 }
@@ -1233,7 +1388,7 @@ ephy_tab_get_security_level (EphyTab *tab)
 static void
 ephy_tab_set_zoom (EphyTab *tab, float zoom)
 {
-	g_return_if_fail (IS_EPHY_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	tab->priv->zoom = zoom;
 
@@ -1243,7 +1398,7 @@ ephy_tab_set_zoom (EphyTab *tab, float zoom)
 float
 ephy_tab_get_zoom (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), 1.0);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), 1.0);
 
 	return tab->priv->zoom;
 }
@@ -1251,7 +1406,7 @@ ephy_tab_get_zoom (EphyTab *tab)
 GObject *
 ephy_tab_get_action (EphyTab *tab)
 {
-	g_return_val_if_fail (IS_EPHY_TAB (tab), NULL);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
 	return G_OBJECT (tab->priv->action);
 }
