@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: statusbar.c,v 1.7 2003/09/09 08:27:05 chpe Exp $
+ *  $Id: statusbar.c,v 1.8 2003/12/13 17:47:10 marco Exp $
  */
 
 #include "statusbar.h"
@@ -193,22 +193,3 @@ statusbar_set_progress (Statusbar *t,
 					       tmp);
 	}
 }
-
-void
-statusbar_set_message (Statusbar *s,
-		       const char *message)
-{
-	char *tmp1, *tmp2;
-
-	g_return_if_fail (message != NULL);
-
-	tmp1 = ephy_string_strip_chr (message, '\r');
-	tmp2 = ephy_string_strip_chr (tmp1, '\n');
-
-	gtk_statusbar_pop (GTK_STATUSBAR (s), 0);
-	gtk_statusbar_push (GTK_STATUSBAR (s), 0, tmp2);
-
-	g_free (tmp1);
-	g_free (tmp2);
-}
-
