@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.100 2003/12/07 10:23:31 chpe Exp $
+ *  $Id: window-commands.c,v 1.102 2003/12/25 14:48:12 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -852,7 +852,7 @@ window_cmd_edit_toolbar (GtkAction *action,
 {
 	GtkWidget *editor;
 	EphyToolbarsModel *model;
-	Toolbar *t;
+	GtkWidget *t;
 	GtkWidget *dialog;
 
 	model = EPHY_TOOLBARS_MODEL
@@ -921,7 +921,7 @@ window_cmd_help_about (GtkAction *action,
 
 	static char *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
-		"Xan Lopez <xan@masilla.org>",
+		"Xan Lopez <xan@gnome.org>",
 		"David Bordoley <bordoley@msu.edu>",
 		"Christian Persch <chpe@gnome.org>",
 		NULL
@@ -1095,7 +1095,7 @@ window_cmd_load_location (GtkAction *action,
 	Toolbar *toolbar;
 	const char *location;
 
-	toolbar = ephy_window_get_toolbar (window);
+	toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
 	location = toolbar_get_location (toolbar);
 
 	if (location)
